﻿

***************SAS macro for implementation of estimating interval effect in terms of a fold-increase endpoint with zero-inflated paired interval-censored data;
%macro est_pairedIC(indata=, subjectID=, group=, logL_T0=, logU_T0=, logL_T1=, logU_T1=, logb1=, logD=, outdata=, seedboot=123, bootn=50, mcQ=1000000, seedmc=234);
ods noresults;
options nonotes nosource nosource2 errors=0;
/********************************************************************
Compulsory parameters:

indata     = name of the sas data file 
subjectID  = name of the variable for identifying the subjects
group      = name of the indicator variable for the treatment allocations (0=control; 1=intervention)
logL_T0    = name of the variable for lower boundary logL of the censoring interval [logL, logU] at T0 (e.g. baseline). If logL_T0=. then treated as left-censoring.  
logU_T0    = name of the variable for upper boundary logU of the censoring interval [logL, logU] at T0 (e.g. baseline). If logU_T0=. then treated as right-censoring.   
logL_T1    = name of the variable for lower boundary logL of the censoring interval [logL, logU] at T1 (e.g. end-of-study). If logL_T1=. then treated as left-censoring.  
logU_T1    = name of the variable for upper boundary logU of the censoring interval [logL, logU] at T1 (e.g. end-of-study). If logU_T1=. then treated as right-censoring.  
logb1      = minimum required value at T1 (after log-transformation) in order to meet the response criteria
logD       = cut-off for a response on the log-transformed scale
outdata    = name of the sas data file to store the parameter estimates for both the analysis data set [indexed by replicate=0] and the bootstrap samples


Optional parameters:

seedboot   = seed for bootstrap resampling (default=123)
bootn      = number of bootstrap samples   (default=50)
mcQ        = number of bivariate normal variates for the Monte Carlo approximation for the intergration in expression 6 (default=1000000)
seedmc     = seed for generating the bivariate normal variates (default=234)
*********************************************************************/


*******************Step 1: estimation of the model parameters;
%put  **********Step 1: Estimation of the model parameters;
/*generate bootstrap samples, and then combine them with the mother data set into a single data set*/
proc surveyselect
     data=&indata         
     out=bootdata         
     method=urs       
     samprate=100     
     rep=&bootn      
     outhits          
     seed=&seedboot; 
run;

proc sort data=bootdata;
by replicate &subjectID;
run;

data bootdata2;
set bootdata;
lower0=&logL_T0;
upper0=&logU_T0;
lower1=&logL_T1;
upper1=&logU_T1;
keep replicate &group lower0 upper0 lower1 upper1;
run;

data bootdata2;
set bootdata2;
&subjectID+1;
by replicate;
if first.replicate then &subjectID=1;
run;

data mergedata;
set &indata bootdata2;
if replicate=. then replicate=0;
run;

proc sort data=mergedata;
by replicate &subjectID &group;
run;

proc transpose data=mergedata out=tmplower prefix=lower;
var lower0 lower1;
by replicate &subjectID &group;
run;  

proc transpose data=mergedata out=tmpupper prefix=upper;
var upper0 upper1;
by replicate &subjectID &group;
run;  
  

data whole;
merge tmplower (rename=(lower1=lower) drop=_name_) 
tmpupper (rename=(upper1=upper));
by replicate &subjectID;
time=input(substr(_name_, 6),5.);
drop _name_;
if time=0 then &group=0;
if lower=. & upper~=.   then censor=0;
if lower~=. & upper~=.  then censor=1;
if lower~=. & upper=.   then censor=2;
run;


*caculating initial parameter estimates by fitting a mdoel without random effects;
ods listing close;
ods output ParameterEstimates=initpar; 

proc nlmixed data=whole;
  by replicate;
	lnteta=alpha0+alpha1*&group+alpha2*time;
	teta=exp(lnteta);
	mu=beta0+beta1*&group+beta2*time;
	lnsigma=delta0+delta1*&group+delta2*time;
	sigma=exp(lnsigma);

	if censor=0        then ll=-log(1+teta)+log(1+teta*probnorm((upper-mu)/sigma));
	if censor=1        then ll=lnteta-log(1+teta)+log(probnorm((upper-mu)/sigma)-probnorm((lower-mu)/sigma));
	if censor=2        then ll=lnteta-log(1+teta)+log(probnorm((mu-lower)/sigma));

    model censor ~general(ll);
run;

data initpar;
set initpar;
keep replicate parameter estimate;
run;

proc transpose data=initpar out=trans_init;
var estimate;
id  parameter;
by replicate;
run;

data trans_init;
set trans_init;
drop _NAME_;
R11=1;
R12=0;
R22=1;
run;

*fitting the mdoel with random effects*;
ods output ParameterEstimates=parm1;
ods output ConvergenceStatus=conv;
proc nlmixed data=whole;
by replicate;
	parms /bydata DATA=trans_init;

	lnteta=alpha0+alpha1*&group+alpha2*time+R11*z_1;
	teta=exp(lnteta);
	mu=beta0+beta1*&group+beta2*time+R12*z_1+R22*z_2;
	lnsigma=delta0+delta1*&group+delta2*time;
	sigma=exp(lnsigma);

	if censor=0        then ll=-log(1+teta)+log(1+teta*probnorm((upper-mu)/sigma));
	if censor=1        then ll=lnteta-log(1+teta)+log(probnorm((upper-mu)/sigma)-probnorm((lower-mu)/sigma));
	if censor=2        then ll=lnteta-log(1+teta)+log(probnorm((mu-lower)/sigma));

	model censor ~general(ll);
	random z_1 z_2 ~ normal([0,0],[1,0,1]) subject=&subjectID;
    run;

	proc sort data=parm1;
	by replicate;
    run;

    proc sort data=conv;
	by replicate;
    run;

    proc transpose data=parm1 out=parm2;
    var estimate;
    id  parameter;
    by replicate;
    run;

	data outtmp;
	merge parm2 conv;
	by replicate;
	drop _NAME_;
	run; 

/********************Step 2: estimation of the risk parameters and efficacy parameters, based on Monte Carlo approximation method;*/
%put *******************Step 2: Estimation of the risk parameters and efficacy parameters via Monte Carlo approximation;
%put Generating standard binormal variates...... ;    
    %let Qd=%sysfunc(floor(&mcQ/20000));
	%let Qr=%sysfunc(ceil(%sysfunc(mod(&mcQ,20000))/2));

	data seed;
     call streaminit(&seedmc);
	 %do i=0 %to &Qd;
	   seedh=round(RAND("UNIFORM")*10000000);
	   qn=&i.;
	   output;
	 %end;
	run;

   %if &Qr>0 %then %do; 
	 data seed;
	  set seed;
	  if qn=0 then call symputx('seedh',seedh); 
	 run;

     data mc_tmp0;
     call streaminit(&seedh);
	  %do i = 1 %to &Qr; 
        v1=RAND('NORMAL');
	    v2=RAND('NORMAL'); 
		mergeID=1;
	    output; 
	   %end;
	  run;
    %end; 

	%if &Qd>0 %then %do;
	     %do i = 1 %to &Qd; 
              %let ii=%eval(&i.);
			  data seed;
	  			set seed;
	  			if qn=&ii then call symputx('seedh',seedh); 
			  run;

              data mc_tmp1;
	             call streaminit(&seedh); 
	             %do j = 1 %to 10000; 
           			  v1=RAND('NORMAL');
	        		  v2=RAND('NORMAL'); 
					  mergeID=1;
	         		  output; 
	         	 %end;
	           run;

			   %if (&Qr=0) and (&ii=1) %then %do;
			   data mc_tmp0;
			    set mc_tmp1;
			   run;
			   %end;
			   %else %do;
                  proc append base=mc_tmp0 data=mc_tmp1; 
                  run;
			   %end;

		 %end;
	%end;

	data mc_tmp0r;
	  set mc_tmp0;
	  v1=-v1;
	  v2=-v2;
    run;
    proc append base=mc_tmp0 data=mc_tmp0r; 
    run;

%put Monte Carlo Approximation......;

   %let dsid=%sysfunc(open(outtmp));
   %let nobs=%sysfunc(attrn(&dsid,nlobs));
   %let rc=%sysfunc(close(&dsid));

   %if &nobs>0 %then %do;
 	  data outtmp1;
  	   set outtmp;
    	 theta0=.;
    	 theta1=.;
		 ROWID=_N_;
		 mergeID=1;
      run;
    
	  %do i=1 %to &nobs;
          %let ii=%eval(&i.);
          data mc_tmp2;
          set  outtmp1;
          where ROWID=&ii;
          keep alpha0 alpha1 alpha2 beta0 beta1 beta2 delta0 delta1 delta2 R11 R12 R22 mergeID;
          run;

          data mc_cmb;
          merge mc_tmp2 mc_tmp0;
          by mergeID;
          run;
          
		  **;
          data mc_cmb;
		  set mc_cmb;
          w1=R11*v1;
          w2=R12*v1+R22*v2;
               
          **control group;
		  p1ctr=exp(alpha0+w1)/(1+exp(alpha0+w1));
		  p2ctr=exp(alpha0+alpha2+w1)/(1+exp(alpha0+alpha2+w1));
          F1ctr=p2ctr*probnorm((beta0+beta2+w2-&logb1)/exp(delta0+delta2));
		  tmp1ctr=(&logD-beta2)/sqrt(exp(2*delta0)+exp(2*(delta0+delta2)));
		  tmp2ctr=(beta0+beta2+w2-&logb1)/exp(delta0+delta2);
		  tmp3ctr=-exp(delta0+delta2)/sqrt(exp(2*delta0)+exp(2*(delta0+delta2)));
		  theta0=F1ctr-p1ctr*p2ctr*probbnrm(tmp1ctr,tmp2ctr,tmp3ctr);

		  **intervention group;
		  p1int=exp(alpha0+w1)/(1+exp(alpha0+w1));
		  p2int=exp(alpha0+alpha1+alpha2+w1)/(1+exp(alpha0+alpha1+alpha2+w1));
          F1int=p2int*probnorm((beta0+beta1+beta2+w2-&logb1)/exp(delta0+delta1+delta2));
		  tmp1int=(&logD-beta1-beta2)/sqrt(exp(2*delta0)+exp(2*(delta0+delta1+delta2)));
		  tmp2int=(beta0+beta1+beta2+w2-&logb1)/exp(delta0+delta1+delta2);
		  tmp3int=-exp(delta0+delta1+delta2)/sqrt(exp(2*delta0)+exp(2*(delta0+delta1+delta2)));
		  theta1=F1int-p1int*p2int*probbnrm(tmp1int,tmp2int,tmp3int);
		  run;
			    
          proc summary data=mc_cmb;
		    var theta0 theta1;
			output out=mc_avg mean=;
			run;

		  data mc_avg;
		  set mc_avg;
		  ROWID=&ii;
		  keep theta0 theta1 ROWID;
		  run;

	      %if  (&ii=1) %then %do;
			   data mc_avg_master;
			    set mc_avg;
		       run;
		  %end;
		  %else %do;
                  proc append base=mc_avg_master data=mc_avg;
                  run;
		  %end;
            
	  %end;
      
       data mc_result;
         merge outtmp1 mc_avg_master;
          by ROWID;
		  drop Reason Status mergeID ROWID;
		  RR=theta1/theta0;
		  RD=theta0-theta1;
       run;
	   
       data &outdata;
         set mc_result;
		 drop mergeID ROWID;
	  run;
   %end;

    
%mend;
